unit F_MessingAbout;

(*
Notes on this program...
- the UserInterface does not use any of Delphi's UI sophistication, eg actions,
  so that you can concentrate on the effects of the code rather than on what the UI is doing
- the fonts on the panels for the UI are set to fsBold, purely for legibility when projected at BUG
- not all properties/methods/values are shown here, but they usually follow the same principles
- anything in brackets is not part of TCanvas, but a specially-written utility routine
*)


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  StdCtrls, ExtCtrls, Spin, ComCtrls, ColorGrd;


type
  TfrmMessingAbout = class(TForm)
    pnlInitialStuff: TPanel;
    btnFromColour: TButton;
    shpFromColour: TShape;
    btnToColour: TButton;
    shpToColour: TShape;
    btnSolidClear: TButton;
    btnGraduClear: TButton;
    btnLine: TButton;
    spinFromX: TSpinEdit;
    spinFromY: TSpinEdit;
    spinToX: TSpinEdit;
    spinToY: TSpinEdit;
    ckbDot: TCheckBox;
    ckbArrow: TCheckBox;
    btnStatusBar: TButton;
    pnlHeader: TPanel;
    ckbSolid: TCheckBox;
    btnLines: TButton;
    btnPyramidClear: TButton;
    Label1: TLabel;
    Label2: TLabel;
    procedure btnFromColourClick(Sender: TObject);
    procedure btnSolidClearClick(Sender: TObject);
    procedure btnToColourClick(Sender: TObject);
    procedure btnGraduClearClick(Sender: TObject);
    procedure btnLineClick(Sender: TObject);
    procedure btnStatusBarClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnLinesClick(Sender: TObject);
    procedure btnPyramidClearClick(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMessingAbout: TfrmMessingAbout;

implementation

uses
  BugColourControl,
  BugCommon;

{$R *.DFM}

{.$.R bug.res}
// remove the dots if you want to load the bug logo (for the header panel) from this bound-in resource

// an example of a 'cracker' class, which simple redeclares a property as public instead of private
type PanelCanvas = class(TPanel)
     public
       property Canvas;
     end;

var
  bmpLogo: TBitmap;
  TextSize: TSize;
  MarqueeRect: TRect;
  LinePoints: array of TPoint;
  bmpUndo,            // this could be a TImageList, which would allow multiple levels of undo
  bmpOffscreen,
  bmpBrush,
  bmpPaste: TBitmap;  // only assigned if actually pasting something
  bccBackground, bccForeground: TBugColourControl;
  bmpHeaderOffScreen : TBitmap;

procedure TfrmMessingAbout.btnFromColourClick(Sender: TObject);
begin
  shpFromColour.Brush.Color := AnotherColour;
end;

procedure TfrmMessingAbout.btnToColourClick(Sender: TObject);
begin
  shpToColour.Brush.Color := AnotherColour;
end;

procedure TfrmMessingAbout.btnSolidClearClick(Sender: TObject);
begin
//  ClearRectangle(Self.Canvas, Self.ClientRect, shpFromColour.Brush.Color);
  Self.Canvas.Brush.Color := shpFromColour.Brush.Color;
  Self.Canvas.FillRect(Self.ClientRect);

end;

procedure TfrmMessingAbout.btnGraduClearClick(Sender: TObject);
begin
  GraduClearRectangle(Self.Canvas, Self.ClientRect, shpFromColour.Brush.Color, shpToColour.Brush.Color);
end;

procedure TfrmMessingAbout.btnLineClick(Sender: TObject);
begin
  btnSolidClearClick(Sender);
  Self.Canvas.Brush.Color := shpFromColour.Brush.Color;
  Self.Canvas.Pen.Color := shpToColour.Brush.Color;
  LineWithEnds(Self.Canvas, Point(spinFromX.Value, spinFromY.Value), Point(spinToX.Value, spinToY.Value), ckbDot.Checked, ckbArrow.Checked, ckbSolid.Checked);
end;

procedure TfrmMessingAbout.btnStatusBarClick(Sender: TObject);
var
  LeftRect, RightRect, BorderRect: TRect;
begin
  with pnlHeader.ClientRect
  do LeftRect := Rect(Left, Top, Left + ((Right - Left) div 2), Bottom);
  with pnlHeader.ClientRect
  do RightRect := Rect(Succ(LeftRect.Right), Top, Right, Bottom);
  with pnlHeader.ClientRect
  do BorderRect := Rect(Left + 2, Top + 2, Right - 2, Bottom - 2);
  if Assigned(bccBackground)
  then begin
//         GraduClearRectangle(PanelCanvas(pnlHeader).Canvas, LeftRect, bccBackground.Colour, bccForeground.Colour, True {lefttoright});
//         GraduClearRectangle(PanelCanvas(pnlHeader).Canvas, RightRect, bccForeground.Colour, bccBackground.Colour, True {lefttoright});
         GraduClearRectangle(bmpHeaderOffscreen.Canvas, LeftRect, bccBackground.Colour, bccForeground.Colour, True {lefttoright});
         GraduClearRectangle(bmpHeaderOffscreen.Canvas, RightRect, bccForeground.Colour, bccBackground.Colour, True {lefttoright});
       end
  else begin
         GraduClearRectangle(PanelCanvas(pnlHeader).Canvas, LeftRect, shpFromColour.Brush.Color, shpToColour.Brush.Color, True {lefttoright});
         GraduClearRectangle(PanelCanvas(pnlHeader).Canvas, RightRect, shpToColour.Brush.Color, shpFromColour.Brush.Color, True {lefttoright});
       end;
//  PanelCanvas(pnlHeader).Canvas.Draw(pnlHeader.Width - 85, 1, bmpLogo);
  bmpHeaderOffscreen.Canvas.Draw(pnlHeader.Width - 85, 1, bmpLogo);
  PanelCanvas(pnlHeader).Canvas.Draw(0,0, bmpHeaderOffscreen);


//  BugBorder(PanelCanvas(pnlHeader).Canvas, BorderRect);
end;


procedure TfrmMessingAbout.FormCreate(Sender: TObject);
begin
  // creation of 2 demo components in place of 2 buttons to change colours
  bccBackground:=TBugColourControl.Create(Self);
  bccBackground.Parent := pnlInitialStuff;
  bccBackground.SetBounds(85, 37, 28, 23);
  bccForeground:=TBugColourControl.Create(Self);
  bccForeground.Parent := pnlInitialStuff;
  bccForeground.SetBounds(211, 37, 28, 23);

  // set up an in-memory bitmap to hold the logo that will go in the header panel
  bmpLogo:= TBitmap.Create;
  bmpLogo.LoadFromFile('Bug_80x40.bmp'); // if you want to load a bitmap from an external BMP-file
//  bmpLogo.LoadFromResourceName(hinstance, 'bug'); // if you want to load a bitmap from a bound-in resource
  bmpLogo.Transparent:= True; // sets the Transparent colour to that of Bottom-Left pixel

  bmpHeaderOffScreen:= TBitmap.Create;
end;

procedure TfrmMessingAbout.FormDestroy(Sender: TObject);
begin
  bmpLogo.Free;
  bmpHeaderOffScreen.Free;
  LinePoints:= nil;  // the equivalent of free for a dynamic array
end;

procedure TfrmMessingAbout.btnLinesClick(Sender: TObject);
begin
  SomeLines(Self.Canvas);
end;

procedure TfrmMessingAbout.btnPyramidClearClick(Sender: TObject);
begin
  PyramidClearRectangle(Self.Canvas, Self.ClientRect, shpFromColour.Brush.Color, shpToColour.Brush.Color);
//  PyramidClearRectangle(PanelCanvas(pnlHeader).Canvas, pnlHeader.ClientRect, shpFromColour.Brush.Color, shpToColour.Brush.Color);
end;

procedure TfrmMessingAbout.FormPaint(Sender: TObject);
begin
btnSolidClearClick(Self);
end;

procedure TfrmMessingAbout.FormResize(Sender: TObject);
begin
  bmpHeaderOffscreen.width := pnlHeader.Width;
  bmpHeaderOffscreen.height := pnlHeader.height;
  btnStatusBarClick(Sender);
end;

end.


